@@ -1505,10 +1505,9 @@ static struct extent_buffer *alloc_dummy_eb(u64 bytenr, u32 size)
 {
 	struct extent_buffer *eb;
 
-	eb = malloc(sizeof(struct extent_buffer) + size);
+	eb = calloc(1, sizeof(struct extent_buffer) + size);
 	if (!eb)
 		return NULL;
-	memset(eb, 0, sizeof(struct extent_buffer) + size);
 
 	eb->start = bytenr;
 	eb->len = size;
@@ -2788,9 +2787,11 @@ int main(int argc, char *argv[])
 
 	if (compress_level > 0 || create == 0) {
 		if (num_threads == 0) {
-			num_threads = sysconf(_SC_NPROCESSORS_ONLN);
-			if (num_threads <= 0)
-				num_threads = 1;
+			long tmp = sysconf(_SC_NPROCESSORS_ONLN);
+
+			if (tmp <= 0)
+				tmp = 1;
+			num_threads = tmp;
 		}
 	} else {
 		num_threads = 0;
